/*******************************************************************************
 * Part of "Intel(R) Active Management Technology (Intel(R) AMT)
 *                   User Notification Service (UNS)"
 *
 * Copyright (c) 2007 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "AMTStatusThread.h"
#include "Lock.h"
#include <sstream>
#include <string>

const unsigned long AMTStatusThread::DEFAULT_CHECK_AMT_STATUS_TIMEOUT = 10 * SECOND;
const unsigned long AMTStatusThread::HECI_DOWNTIME_ON_RESUME = 10 * SECOND;

// Constructor
AMTStatusThread::AMTStatusThread(bool verbose, UNSEventLogger *pLogger /*= NULL*/) :
PeriodicActionThread("AMTStatusCheck", DEFAULT_CHECK_AMT_STATUS_TIMEOUT, HECI_DOWNTIME_ON_RESUME),
m_AMTStatusReader(verbose)
{
	m_init = false;
	m_iATStatus = NULL;
	m_pLogger = pLogger;
	m_nNumPollsDisabled = 0;
	m_State = INITIAL_STATE;
	m_AMTStatusReader.SetLogger(pLogger);
	m_pSemIsAMTEnabled = NULL;
	m_pIsAMTEnabled = NULL;
}

//Destructor
AMTStatusThread::~AMTStatusThread()
{
	m_iATStatus = NULL;
	m_pSemIsAMTEnabled = NULL;
	m_pIsAMTEnabled = NULL;
}

// Initialize this thread with a given action interval time.
bool AMTStatusThread::init(const UNSConfig &Config, ATStatus &iATStatus)
{
	m_UNSConfig = Config;

	m_iATStatus = &iATStatus;

	if (!m_AMTStatusReader.init(Config)) {
		return false;
	}

	if (iATStatus.IsAmtKnown()) {
		m_AMTStatusReader.SetAMTDisabled(!iATStatus.IsAmtEnabled());
	}

	setActionInterval(m_UNSConfig.CheckAMTStatusTimeout());

	m_init = true;

	return true;
}

void AMTStatusThread::SetLogger(UNSEventLogger *pLogger)
{
	m_pLogger = pLogger;
	m_AMTStatusReader.SetLogger(pLogger);
}

void AMTStatusThread::debugLog(const char *msg)
{
	if (m_pLogger && m_UNSConfig.TraceLogEnabled()) {
		m_pLogger->DebugLog(msg);
	}
}

void AMTStatusThread::SetParamsAMTEnabled(Semaphore *semAMTEnabled, bool *amtEnabled)
{
	m_pSemIsAMTEnabled = semAMTEnabled;
	m_pIsAMTEnabled = amtEnabled;
}

AMTStatusThread::State AMTStatusThread::SetAMTData(bool bLogging /*= false*/)
{
	if ((m_pSemIsAMTEnabled != NULL) && (m_pIsAMTEnabled != NULL)) {
		Lock enaLock(*m_pSemIsAMTEnabled);
		*m_pIsAMTEnabled = (AMTSTATE_POST == m_AMTStatusData.amtState);
	}

	if (m_iATStatus) {
		m_iATStatus->SetAMTData(m_AMTStatusData, bLogging);

		if (m_UNSConfig.TraceLogEnabled()) {
			std::stringstream ss;
			ss << "Setting AMT status: " << ((AMTSTATE_POST == m_AMTStatusData.amtState) ? "Enabled"
							: ((AMTSTATE_NOT_DETECTED == m_AMTStatusData.amtState) ? "Unknown"
							: "Disabled"));
			debugLog(ss.str().c_str());
		}
	}

	return ((AMTSTATE_POST == m_AMTStatusData.amtState) ? ENABLED_STATE : DISABLED_STATE);
}


void AMTStatusThread::action()
{
	if (!m_init) {
		return;
	}

	switch (m_State)
	{
	case INITIAL_STATE:
		m_State = OnInitialState();
		break;
	case ENABLED_STATE:
		m_State = OnEnabledState();
		break;
	case POSSIBLY_DISABLED_STATE:
		m_State = OnPossiblyDisabledState();
		break;
	case DISABLED_STATE:
		m_State = OnDisabledState();
		break;
	}
}

void AMTStatusThread::cleanup()
{
}

AMTStatusThread::State AMTStatusThread::OnInitialState()
{
	if (!(m_AMTStatusReader.GetAMTStatusData(m_AMTStatusData))) { //not use FWU - SetAMTDisabled called from init
		if (m_pLogger) {
			m_pLogger->HECICommandFailedError();
		}
	}
	return SetAMTData(true);
}

AMTStatusThread::State AMTStatusThread::OnEnabledState()
{
	if ((m_AMTStatusReader.GetAMTStatusData(m_AMTStatusData))
	    && (AMTSTATE_POST == m_AMTStatusData.amtState)) {
		return SetAMTData();
	}
	else {
		m_nNumPollsDisabled = 1; //We got "disabled" status from HECI once.
		return OnPossiblyDisabledState(false);
	}
}

AMTStatusThread::State AMTStatusThread::OnPossiblyDisabledState(bool bNeedCheck /*=true*/)
{
	//bNeedCheck tells us if we need to check the status by HECI. When it's false,
	//the status received from HECI is assumed to be disabled.
	if (bNeedCheck
	    && ((m_AMTStatusReader.GetAMTStatusData(m_AMTStatusData))
		&& (AMTSTATE_POST == m_AMTStatusData.amtState)) ) {
		return SetAMTData();
	}
	else {
		if (m_nNumPollsDisabled++ >= m_UNSConfig.NumPollsForAMTDisabled()) {
			//We reached the threshold for setting "disabled" status.
			if (m_UNSConfig.TraceLogEnabled()) {
				std::stringstream ss;
				ss << "Got 'disabled' status from HECI " << (m_nNumPollsDisabled-1) << " times."
					<< " Setting AMT status to 'disabled'.";
				debugLog(ss.str().c_str());
			}
			return SetAMTData();
		}
		else {
			//We haven't reached the threshold yet.
			if (m_UNSConfig.TraceLogEnabled()) {
				std::stringstream ss;
				ss << "Got 'disabled' status from HECI " << (m_nNumPollsDisabled-1) << " times."
					<< " Still not changing AMT status to 'disabled'.";
				debugLog(ss.str().c_str());
			}
			return POSSIBLY_DISABLED_STATE;
		}
	}
}

AMTStatusThread::State AMTStatusThread::OnDisabledState()
{
	m_AMTStatusReader.GetAMTStatusData(m_AMTStatusData);
	return SetAMTData();
}
