/*******************************************************************************
 * Part of "Intel(R) Active Management Technology (Intel(R) AMT)
 *                   User Notification Service (UNS)"
 *
 * Copyright (c) 2007 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "ATStatus.h"
#include "Lock.h"

#include <sstream>
#include <string>

#define MEFWCAPS_MANAGEABILITY_SUPP_UNKNOWN 0xDD

// Constructor
ATStatus::ATStatus(bool verbose, UNSEventLogger *pLogger /*= NULL*/) :
m_fwulCommand(verbose),
m_mngCommand(verbose)
{
	m_pLogger = pLogger;
	m_init = false;
	clearData();
}

//Destructor
ATStatus::~ATStatus()
{
	Deinit();
}

void ATStatus::clearData()
{
	memset(&m_skuBits, 0, sizeof(m_skuBits));
	m_ena = 0;

	memset(&m_amtStatusData, 0, sizeof(AMTStatusData));
	m_amtEvents.clear();
#ifdef WITH_TPM
	memset(&m_tpmStatusData, 0, sizeof(TPMStatusData));
	m_tpmEvents.clear();
#endif
	m_asyncEvents.clear();
}

void ATStatus::Deinit()
{
	Lock lock(m_dataSetSemaphore);

	m_init = false;

	m_IPCiATStatus.close();

	clearData();
}

bool ATStatus::readSkuInfo()
{
	FWU_GET_VERSION_MSG_REPLY verMsg;
	FWU_GET_INFO_MSG_REPLY infoMsg;
	MNG_GET_ME_INFORMATION_RESPONSE mngIMsg;
	HECI_STATUS heciRet;
	MEFWCAPS_SKU tmpSku;
	MEFWCAPS_SKU tmpSkuHw;

	if (!m_fwulCommand.FWULClient.Init()) {
		if (m_pLogger) {
			m_pLogger->ErrorLog("SKU check failed: Unable to connect via MEI driver.");
		}
		return false;
	}
	heciRet = m_fwulCommand.GetFWUVersionAndInfo(verMsg, infoMsg);
	m_fwulCommand.FWULClient.Deinit();
	if (HECI_STATUS_OK != heciRet) {
		std::stringstream ss;
		ss << "SKU check failed with code =" << (unsigned int)heciRet;
		if (m_pLogger) {
			m_pLogger->ErrorLog(ss.str().c_str());
		}
		return false;
	}

	tmpSku.Data = verMsg.Sku;
	tmpSkuHw.Data = verMsg.HwSku;

	m_skuBits.Data = (tmpSku.Data & tmpSkuHw.Data);
	if (m_skuBits.Fields.Amt) {
		m_skuBits.Fields.AmtFund = 0;
	} else if (m_skuBits.Fields.AmtFund) {
		m_skuBits.Fields.Amt = 1;
	}

	if (infoMsg.MessageType == FWU_GET_INFO_REPLY) {
		m_ena = infoMsg.ManageabilityMode;

		if (!m_mngCommand.MNGClient.Init()) {
			if (m_pLogger) {
				m_pLogger->WarningLog("MNG SKU check failed: Unable to connect MNG via MEI driver.");
			}
			return true;
		}
		heciRet = m_mngCommand.GetMEInfo(mngIMsg);
		m_mngCommand.MNGClient.Deinit();
		if (HECI_STATUS_OK != heciRet) {
			std::stringstream ss;
			ss << "MNG SKU check failed with code =" << (unsigned int)heciRet;
			if (m_pLogger) {
				m_pLogger->WarningLog(ss.str().c_str());
			}
			return true;
		}

		m_skuBits = mngIMsg.Sku;
		if (!m_skuBits.Fields.Amt) {
			m_skuBits.Fields.AmtFund = 0;
		}

		m_ena = mngIMsg.MngMode;
	} else {
		m_ena = MEFWCAPS_MANAGEABILITY_SUPP_UNKNOWN;
	}
	return true;
}

bool ATStatus::IsAmtSku()
{
	return ((m_skuBits.Fields.Amt) ? true : false);
}

bool ATStatus::IsAmtEnabled()
{
	return (m_ena == MEFWCAPS_MANAGEABILITY_SUPP_AMT);
}

bool ATStatus::IsAmtKnown()
{
	return (m_ena != MEFWCAPS_MANAGEABILITY_SUPP_UNKNOWN);
}

bool ATStatus::IsAmtFundamental()
{
	return ((m_skuBits.Fields.Amt && m_skuBits.Fields.AmtFund) ? true : false);
}

#ifdef WITH_TPM
bool ATStatus::IsTpmSku()
{
	return ((m_skuBits.Fields.Tpm) ? true : false);
}
#endif

// Initialize this thread with a given action interval time.
bool ATStatus::Init()
{
	bool result = true;
	Lock lock(m_dataSetSemaphore);

	clearData();

	m_IPCiATStatus.init(true);

	if (!readSkuInfo()) {
		result = false;
	}

	if (result) {
		if (IsAmtSku()) {
			int aCollIni = m_AMTStatusColl.Init(true);
			if (0 != aCollIni) {
				std::stringstream ss;
				ss << "Intel(R) AMT status init result=" << aCollIni;
				if (m_pLogger) {
					m_pLogger->WarningLog(ss.str().c_str());
				}
			}
		}
#ifdef WITH_TPM
		if (IsTpmSku()) {
			int tCollIni = m_TPMStatusColl.Init(true);
			if (0 != tCollIni) {
				std::stringstream ss;
				ss << "Intel(R) TPM status init result=" << tCollIni;
				if (m_pLogger) {
					m_pLogger->WarningLog(ss.str().c_str());
				}
			}
		}
#endif
	}

	if (!m_IPCiATStatus.isOk()) {
		std::stringstream ss;
		ss << "IPCiATStatus object is not ok: last error=" << m_IPCiATStatus.lastError();
		if (m_pLogger) {
			m_pLogger->ErrorLog(ss.str().c_str());
		}
		result = false;
	}

	m_init = result;
	return result;
}

bool ATStatus::SetAMTData(const AMTStatusData &amtData, bool bLogging /*= false*/)
{
	bool result = true;
	Lock lock(m_dataSetSemaphore);

	if (!m_init) {
		return false;
	}

	m_amtEvents.clear();
	memcpy(&m_amtStatusData, &amtData, sizeof(AMTStatusData));

	if (1 < m_AMTStatusColl.Collate(m_amtStatusData, &m_amtEvents)) {
		//filter duplicate async events
		filterAsyncEvents(m_amtEvents);
		//log Events
		std::list<ATEvents::EventNode>::const_iterator iter = m_amtEvents.begin();
		std::list<ATEvents::EventNode>::const_iterator end = m_amtEvents.end();
		for (; iter != end; iter++) {
			logEvent(*iter);
		}
		//add async events to AMT events
		m_amtEvents.insert(m_amtEvents.begin(), m_asyncEvents.begin(), m_asyncEvents.end());
		m_asyncEvents.clear();
		//alter events history
		if (0 > m_eventHistory.AddEvents(m_amtEvents)) {
			if (bLogging && m_pLogger) {
				m_pLogger->UpdateEventHistoryFailedError();
			}
			result = false;
		}
	}

	//tell privacy application about events and data
	putData();
	if (!m_IPCiATStatus.SetEnablement(enablementFromData())) {
		if (bLogging && m_pLogger) {
			m_pLogger->SetATStatusFailedError();
		}
		result = false;
	}

	return result;
}

#ifdef WITH_TPM
bool ATStatus::SetTPMData(const TPMStatusData &tpmData, bool bLogging /*= false*/)
{
	bool result = true;
	Lock lock(m_dataSetSemaphore);

	if (!m_init) {
		return false;
	}

	m_tpmEvents.clear();
	memcpy(&m_tpmStatusData, &tpmData, sizeof(TPMStatusData));

	if (1 < m_TPMStatusColl.Collate(m_tpmStatusData, &m_tpmEvents)) {
		//log Events
		std::list<ATEvents::EventNode>::const_iterator iter = m_tpmEvents.begin();
		std::list<ATEvents::EventNode>::const_iterator end = m_tpmEvents.end();
		for (; iter != end; iter++) {
			logEvent(*iter);
		}
		//alter events history
		if (0 > m_eventHistory.AddEvents(m_tpmEvents)) {
			if (bLogging && m_pLogger) {
				m_pLogger->UpdateEventHistoryFailedError();
			}
			result = false;
		}
	}

	//tell privacy application about events and data
	putData();
	if (!m_IPCiATStatus.SetEnablement(enablementFromData())) {
		if (bLogging && m_pLogger) {
			m_pLogger->SetATStatusFailedError();
		}
		result = false;
	}

	return result;
}
#endif

//prepare data to set
bool ATStatus::putData()
{
	unsigned int skui = (IsAmtSku() ? IPCiATStatus::SKU_AMT : 0);

	if (IsAmtFundamental()) {
		skui |= IPCiATStatus::SKU_AMTFUND;
	}
#ifdef WITH_TPM
	if (IsTpmSku()) {
		skui |= IPCiATStatus::SKU_TPM;
	}
#endif

	m_IPCiATStatus.PutSku(skui);
	m_IPCiATStatus.PutAMTStatusData(m_amtStatusData);
	m_IPCiATStatus.PutEvents(m_amtEvents);
#ifdef WITH_TPM
	m_IPCiATStatus.PutTPMStatusData(m_tpmStatusData);
	m_IPCiATStatus.AddEvents(m_tpmEvents);
#endif
	m_IPCiATStatus.AddEvents(m_asyncEvents);

	return true;
}

IPCiATStatus::Enablement ATStatus::enablementFromData() const
{
	IPCiATStatus::Enablement enablement;
#ifdef WITH_TPM
	int tpmState = m_TPMStatusColl.TPMOperationalState(m_tpmStatusData);
#endif

	switch (m_amtStatusData.amtState)
	{
	case AMTSTATE_POST:
		enablement = IPCiATStatus::STATUS_ENABLED;
#ifdef WITH_TPM
		switch (tpmState)
		{
		case TPM_COLL_OPER_OPERATIONAL:
			enablement = IPCiATStatus::STATUS_ENABLED;
			break;

		case TPM_COLL_OPER_UNKNOWN:
			enablement = IPCiATStatus::STATUS_AMT_ENABLED;
			break;

		default:
			//no operational
			enablement = IPCiATStatus::STATUS_AMT_NOT_DISABLED;
		}
#endif
		break;

	case AMTSTATE_NOT_DETECTED:
		enablement = IPCiATStatus::STATUS_UNKNOWN;
#ifdef WITH_TPM
		switch (tpmState)
		{
		case TPM_COLL_OPER_OPERATIONAL:
			enablement = IPCiATStatus::STATUS_TPM_ENABLED;
			break;

		case TPM_COLL_OPER_UNKNOWN:
			enablement = IPCiATStatus::STATUS_UNKNOWN;
			break;

		default:
			//no operational
			enablement = IPCiATStatus::STATUS_TPM_DISABLED;
		}
#endif
		break;

	default:
		enablement = IPCiATStatus::STATUS_DISABLED;
#ifdef WITH_TPM
		switch (tpmState)
		{
		case TPM_COLL_OPER_OPERATIONAL:
			enablement = IPCiATStatus::STATUS_TPM_NOT_DISABLED;
			break;

		case TPM_COLL_OPER_UNKNOWN:
			enablement = IPCiATStatus::STATUS_AMT_DISABLED;
			break;

		default:
			//no operational
			enablement = IPCiATStatus::STATUS_DISABLED;
		}
#endif
	}

	return enablement;
}

void ATStatus::logEvent(const ATEvents::EventNode &event)
{
	if (!m_pLogger) {
		return;
	}

	switch (event.m_id)
	{
	case ATEvents::EventNode::PROVISIONED:
		m_pLogger->LogInfoEventAmt(AMT_INFO, AMT_PROVISIONED_EVENT);
		break;

	case ATEvents::EventNode::UNPROVISIONED:
		m_pLogger->LogInfoEventAmt(AMT_INFO, AMT_UNPROVISION_EVENT);
		break;

	case ATEvents::EventNode::SD_STARTED:
		m_pLogger->LogInfoEventAmt(UNS_CIRCUIT_BREAKER, SYSTEM_DEFENSE_STARTED_EVENT);
		break;

	case ATEvents::EventNode::SD_FINISHED:
		m_pLogger->LogInfoEventAmt(UNS_CIRCUIT_BREAKER, SYSTEM_DEFENSE_ENDED_EVENT);
		break;

	case ATEvents::EventNode::REMOTE_REBOOT:
		m_pLogger->LogInfoEventAmt(AMT_INFO, REMOTE_CONTROL_EVENT);
		break;

#ifdef WITH_TPM
	case ATEvents::EventNode::TPM_OPERATIONAL:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_OPERATIONAL_WEVENT);
		break;

	case ATEvents::EventNode::TPM_NOT_DETECTED:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_NOT_DETECTED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_FAIL:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_FAILED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_ACTIVE:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_ACTIVE_WEVENT);
		break;

	case ATEvents::EventNode::TPM_NOT_ACTIVE:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_DEACTIVATED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_OWNED:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_OWNED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_NOT_OWNED:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_NOT_OWNED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_ENABLE:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_ENABLED_WEVENT);
		break;

	case ATEvents::EventNode::TPM_NOT_ENABLE:
		m_pLogger->LogInfoEvent(TPM_INFO, TPM_DISABLED_WEVENT);
		break;
#endif
	}
}

bool ATStatus::AsyncEvent(ATEvents::EventNode event)
{
	Lock lock(m_dataSetSemaphore);

	m_asyncEvents.push_back(event);
	return true;
}


void ATStatus::filterAsyncEvents(std::list<ATEvents::EventNode> &events)
{
	std::list<ATEvents::EventNode>::iterator iter0 = events.begin();
	std::list<ATEvents::EventNode>::iterator end0 = events.end();

	while (iter0 != events.end()) {
		bool edel = false;
		std::list<ATEvents::EventNode>::const_iterator iter1 = m_asyncEvents.begin();
		std::list<ATEvents::EventNode>::const_iterator end1 = m_asyncEvents.end();
		for (; iter1 != end1; iter1++) {
			if (iter0->m_id == iter1->m_id) {
				edel = true;
				break;
			}
		}
		if (edel) {
			iter0 = events.erase(iter0);
			continue;
		}
		iter0++;
	}
}


