/*******************************************************************************
 * Part of "Intel(R) Active Management Technology (Intel(R) AMT)
 *                   User Notification Service (UNS)"
 *
 * Copyright (c) 2007 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef WITH_TPM
#include "TPMStatusThread.h"
#include "TPMStatusColl.h"

#include <sstream>
#include <string>

#define TPM_VALID_VENDOR TPM_VENDOR_INTC

const unsigned long TPMStatusThread::DEFAULT_CHECK_TPM_STATUS_TIMEOUT = 120 * SECOND;
const unsigned long TPMStatusThread::TPM_DOWNTIME_ON_RESUME = 0 * SECOND;

// Constructor
TPMStatusThread::TPMStatusThread(UNSEventLogger *pLogger /*= NULL*/) :
PeriodicActionThread("TPMStatusCheck", DEFAULT_CHECK_TPM_STATUS_TIMEOUT, TPM_DOWNTIME_ON_RESUME),
m_TPMStatusReader()
{
	m_init = false;
	m_iATStatus = NULL;
	m_pLogger = pLogger;
	m_nNumPollsDisabled = 0;
	m_State = INITIAL_STATE;
	m_TPMStatusReader.SetLogger(pLogger);
}

//Destructor
TPMStatusThread::~TPMStatusThread()
{
	m_iATStatus = NULL;
}

// Initialize this thread with a given action interval time.
bool TPMStatusThread::init(const UNSConfig &Config, ATStatus &iATStatus)
{
	m_UNSConfig = Config;

	m_iATStatus = &iATStatus;

	if (!m_TPMStatusReader.init(Config)) {
		return false;
	}

	setActionInterval(m_UNSConfig.CheckTPMStatusTimeout());

	m_init = true;

	return true;
}

void TPMStatusThread::SetLogger(UNSEventLogger *pLogger)
{
	m_pLogger = pLogger;
	m_TPMStatusReader.SetLogger(pLogger);
}


void TPMStatusThread::debugLog(const char *msg)
{
	if (m_pLogger && m_UNSConfig.TraceLogEnabled()) {
		m_pLogger->DebugLog(msg);
	}
}

TPMStatusThread::State TPMStatusThread::SetTPMData(bool bLogging /*= false*/)
{
	int tpmOpState = TPMStatusColl::TPMOperationalState(m_TPMStatusData);

	if (m_iATStatus) {
		m_iATStatus->SetTPMData(m_TPMStatusData, bLogging);

		if (m_UNSConfig.TraceLogEnabled()) {
			std::stringstream ss;
			ss << "Setting TPM status: "
			    << ((TPM_COLL_OPER_OPERATIONAL == tpmOpState) ? "Operational" :
			    ((TPM_COLL_OPER_UNKNOWN == tpmOpState) ? "Unknown" : "Disabled"));
			debugLog(ss.str().c_str());
		}
	}

	return ((TPM_COLL_OPER_OPERATIONAL == tpmOpState) ? ENABLED_STATE : DISABLED_STATE);
}


void TPMStatusThread::action()
{
	if (!m_init) {
		return;
	}

	switch (m_State)
	{
	case INITIAL_STATE:
		m_State = OnInitialState();
		break;
	case ENABLED_STATE:
		m_State = OnEnabledState();
		break;
	case POSSIBLY_DISABLED_STATE:
		m_State = OnPossiblyDisabledState();
		break;
	case DISABLED_STATE:
		m_State = OnDisabledState();
		break;
	}
}

void TPMStatusThread::cleanup()
{
}

TPMStatusThread::State TPMStatusThread::OnInitialState()
{
	if (!(m_TPMStatusReader.GetTPMStatusData(m_TPMStatusData, TPM_VALID_VENDOR))) {
		if (m_pLogger) {
			m_pLogger->TPMCommandFailedError();
		}
	}
	return SetTPMData(true);
}

TPMStatusThread::State TPMStatusThread::OnEnabledState()
{
	if ((m_TPMStatusReader.GetTPMStatusData(m_TPMStatusData, TPM_VALID_VENDOR))
	    && (TPM_COLL_OPER_OPERATIONAL == TPMStatusColl::TPMOperationalState(m_TPMStatusData))) {
		return SetTPMData();
	}
	else {
		m_nNumPollsDisabled = 1; //We got "disabled" status from TPM once.
		return OnPossiblyDisabledState(false);
	}
}

TPMStatusThread::State TPMStatusThread::OnPossiblyDisabledState(bool bNeedCheck /*=true*/)
{
	//bNeedCheck tells us if we need to check the status by TPM. When it's false,
	//the status received from TPM is assumed to be disabled.
	if (bNeedCheck
	    && ((m_TPMStatusReader.GetTPMStatusData(m_TPMStatusData, TPM_VALID_VENDOR))
	    && (TPM_COLL_OPER_OPERATIONAL == TPMStatusColl::TPMOperationalState(m_TPMStatusData))) ) {
		return SetTPMData();
	}
	else {
		if (m_nNumPollsDisabled++ == m_UNSConfig.NumPollsForTPMDisabled()) {
			//We reached the threshold for setting "disabled" status.
			if (m_UNSConfig.TraceLogEnabled()) {
				std::stringstream ss;
				ss << "Got 'not operational' status from TPM " << (m_nNumPollsDisabled-1) << " times."
					<< " Setting TPM status to 'disabled'.";
				debugLog(ss.str().c_str());
			}
			return SetTPMData();
		}
		else {
			//We haven't reached the threshold yet.
			if (m_UNSConfig.TraceLogEnabled()) {
				std::stringstream ss;
				ss << "Got 'non operational' status from TPM " << (m_nNumPollsDisabled-1) << " times."
					<< " Still not changing TPM status to 'disabled'.";
				debugLog(ss.str().c_str());
			}
			return POSSIBLY_DISABLED_STATE;
		}
	}
}

TPMStatusThread::State TPMStatusThread::OnDisabledState()
{
	m_TPMStatusReader.GetTPMStatusData(m_TPMStatusData, TPM_VALID_VENDOR);
	return SetTPMData();
}

#endif
