/*******************************************************************************
 * Part of "Intel(R) Active Management Technology (Intel(R) AMT)
 *                   User Notification Service (UNS)"
 *
 * Copyright (c) 2007 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *******************************************************************************/

//*****************************************************************************
//
// Class:	 UserNotificationServiceLinux
// Description:	 The main class of the service.
//
//*****************************************************************************
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "UserNotificationServiceLinux.h"
#include "UNSCommonDefs.h"
#include "registryWrapper.h"
#include "Options.h"
#include "UNSConfig.h"
#include "XMLUtils.h"

#define NO_ERROR 0
#define ERROR_INVALID_PARAMETER 1

//*****************************************************************************
// Name			: Constructor
// Description	: Constructor - initializes a new instance of the
//		  		  UserNotificationService class.
//*****************************************************************************
UserNotificationService::UserNotificationService() :
#ifdef _DEBUG_MOST
m_ATStatus(true),
m_AMTStatusThread(true)
#else
m_ATStatus(false),
m_AMTStatusThread(false)
#endif
#ifdef WITH_TPM
, m_TPMStatusThread()
#endif
{
	m_init = false;
}

//*****************************************************************************
// Name			: Destructor
// Description	: Destructor - finish soap.
//*****************************************************************************
UserNotificationService::~UserNotificationService(void)
{
	m_UserNotificationThread.SetATStatus(NULL);
	DeinitLogger();
}

//*****************************************************************************
// Name		: InitLogger
// Description	: initializes logger and shares
//*****************************************************************************
void
UserNotificationService::InitLogger(void)
{
	m_isAMTEnabled = false;
	m_Logger = new UNSEventLogger();
	m_UserNotificationThread.SetLogger(m_Logger);
	// this function passes the thread the state of AMT (and it related mutex)
	m_UserNotificationThread.SetParamsAMTEnabled(&m_semIsAMTEnabled, &m_isAMTEnabled);
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.SetLogger(m_Logger);
	// this function passes the thread the state of AMT (and it related mutex)
	m_PostureRetrievalThread.SetParamsAMTEnabled(&m_semIsAMTEnabled, &m_isAMTEnabled);
#endif
	m_ATStatus.SetLogger(m_Logger);
	m_AMTStatusThread.SetLogger(m_Logger);
	// this function passes the thread the state of AMT (and it related mutex)
	m_AMTStatusThread.SetParamsAMTEnabled(&m_semIsAMTEnabled, &m_isAMTEnabled);
#ifdef WITH_TPM
	m_TPMStatusThread.SetLogger(m_Logger);
#endif
}

//*****************************************************************************
// Name		: DeinitLogger
// Description	: initializes logger and shares
//*****************************************************************************
void
UserNotificationService::DeinitLogger(void)
{
#ifdef WITH_TPM
	m_TPMStatusThread.SetLogger(NULL);
#endif
	m_AMTStatusThread.SetLogger(NULL);
	m_AMTStatusThread.SetParamsAMTEnabled(NULL, NULL);
	m_ATStatus.SetLogger(NULL);
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.SetLogger(NULL);
	m_PostureRetrievalThread.SetParamsAMTEnabled(NULL, NULL);
#endif
	m_UserNotificationThread.SetLogger(NULL);
	m_UserNotificationThread.SetParamsAMTEnabled(NULL, NULL);
	if (m_Logger != NULL) {
		delete m_Logger;
		m_Logger = NULL;
	}
}


//*****************************************************************************
// Name		: SetCredentialsMode
// Description	: This is a service set credentials function that gets called
//		  when the service gets created and have command line arguments
//*****************************************************************************
unsigned long
UserNotificationService::SetCredentialsMode(unsigned long argc,
											const char **argv,
											unsigned long *specificError)
{
	InitLogger();

	if (m_Logger != NULL) {
		m_Logger->DebugLog("Intel(R) AMT User Notification set credentials mode");
	}
	printf ("Intel(R) AMT User Notification set credentials mode\n");
	unsigned long rc = UpdateRegistry(argc, const_cast<const char **>(argv));
	if (m_Logger != NULL) {
		m_Logger->DebugLog("Intel(R) AMT User Notification set credentials done");
	}
	printf ("Done.\n");
	*specificError = rc;
	return UNS_STATUS_UPDATE_REG_MODE;
}

//*****************************************************************************
// Name		: ServiceInitialization
// Description	: This is a service initialization function that gets called
//		  when the service gets created
//*****************************************************************************
unsigned long
UserNotificationService::ServiceInitialization(unsigned long argc,
											   const char **argv,
											   unsigned long *specificError)
{
	UNSConfig Config;

	InitLogger();

	/* Initialize Configuration object */
	Config.Init();

#ifdef _DEBUG
#ifndef _DEBUG_MOST
	if (!uns_debug_mode) {
		uns_debug_mode = Config.DebugMode();
	}
#endif
#endif

	CimXMLUtilsNamespace::InitXMLLibrary();

	// Initialize the threads.
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.init(Config);
#endif
	m_UserNotificationThread.init();

	if (m_ATStatus.Init()) {
		if (m_ATStatus.IsAmtSku()) {
			if (!m_AMTStatusThread.init(Config, m_ATStatus)) {
				// never happens
				if (m_Logger != NULL) {
					m_Logger->ErrorLog("Intel(R) AMT Status thread initialization failed");
				}
			}
		} else {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Intel(R) AMT not present on SKU");
			}
		}
#ifdef WITH_TPM
		if (m_ATStatus.IsTpmSku()) {
			if (!m_TPMStatusThread.init(Config, m_ATStatus)) {
				// never happens
				if (m_Logger != NULL) {
					m_Logger->ErrorLog("Intel(R) TPM Status thread initialization failed");
				}
			}
		} else {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Intel(R) TPM not present on SKU");
			}
		}
#endif
		m_UserNotificationThread.SetATStatus(&m_ATStatus);
	}

	m_init = true;

	return UNS_STATUS_SUCCESS;
}

//*****************************************************************************
// Name		:	Stop
// Description	:	Initiates the stop service action.
//*****************************************************************************
void UserNotificationService::Stop()
{
	m_StopSemaphore.release();
}

//*****************************************************************************
// Name		:	InternalStop
// Description	:	Stop the service.
//*****************************************************************************
void UserNotificationService::InternalStop()
{
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.stop();
#endif
	m_UserNotificationThread.stop();

	if (m_ATStatus.IsAmtSku()) {
		m_AMTStatusThread.stop();
	}
#ifdef WITH_TPM
	if (m_ATStatus.IsTpmSku()) {
		m_TPMStatusThread.stop();
	}
#endif
	m_ATStatus.Deinit();

	if (m_Logger != NULL) {
		m_Logger->ServiceStopped();	// Record stop in event log
	}

	CimXMLUtilsNamespace::TerminateXMLLibrary();
}

//*****************************************************************************
// Name		: start
// Description	: Listen of the master socket, if a data is received from the
//		  TCP network process the data: Invoke soap_serve(&soap):
//		  The soap_serve() function will invoke the __emc__SoapAlert.
//
//*****************************************************************************
unsigned long UserNotificationService::Start()
{
	if (m_Logger != NULL) {
		m_Logger->ServiceStart();
	}
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.start();
#endif
	m_UserNotificationThread.start();

	if (m_ATStatus.IsAmtSku()) {
		m_AMTStatusThread.start();
	}
#ifdef WITH_TPM
	if (m_ATStatus.IsTpmSku()) {
		m_TPMStatusThread.start();
	}
#endif

	m_StopSemaphore.acquire();
	//Check for releasing the semaphore
	while (!m_StopSemaphore.acquireTry()) {
		usleep(200);
	}

	InternalStop();

	return UNS_STATUS_SUCCESS;
}

//*****************************************************************************
// Name		:	UpdateRegistry
// Description	:	Run the programm in update registry mode.
//*****************************************************************************
unsigned long UserNotificationService::UpdateRegistry(int argc, const char **argv)
{
	Credentials *unsCreds = NULL;
#ifdef WITH_AMTPP
	Credentials *eacCreds = NULL;
#endif
	unsigned long res;
	bool clearRegistry = false;

	res = ParseArguments(argc, argv, &unsCreds
#ifdef WITH_AMTPP
			     , &eacCreds
#endif
			     , clearRegistry);
	if (res != NO_ERROR) {
		// since this is an error that aborts the service initialization.
		if (m_Logger != NULL) {
			m_Logger->ErrorLog("Attempted to start the service with bad arguments.");
		}
		printf("\nAttempted to start the service with bad arguments.\n");
		return res;
	}

	if (clearRegistry) {
#ifdef WITH_AMTPP
		printf("\nRemoving Endpoint Access Control credentials...\n");
		if (!DeleteCredentials(CRED_REG_INFO::EAC)) {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Failed to remove EAC credentials");
			}
			printf("\nFailed to remove EAC credentials\n");
		} else {
			if (m_Logger != NULL) {
				m_Logger->DebugLog("Endpoint Access Control credentials were removed");
			}
		}
#endif
		printf("\nRemoving User Notification credentials...\n");
		if (!DeleteCredentials(CRED_REG_INFO::UNS)) {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Failed to remove UNS credentials");
			}
			printf("\nFailed to remove UNS credentials\n");
		} else {
			if (m_Logger != NULL) {
				m_Logger->DebugLog("User Notification credentials were removed");
			}
		}
		return 0;
	}

#ifdef WITH_AMTPP
	if (eacCreds) {
		printf("\nUpdating Endpoint Access Control credentials...\n");
		// store in registry
		if (!StoreCredentials(eacCreds, CRED_REG_INFO::EAC)) {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Failed to store EAC credentials");
			}
			printf("\nFailed to store EAC credentials\n");
		} else {
			if (m_Logger != NULL) {
				m_Logger->DebugLog("Endpoint Access Control credentials were updated");
			}
		}
		delete eacCreds;
	}
#endif

	if (unsCreds) {
		printf("\nUpdating User Notification credentials...\n");

		// store in registry
		if (!StoreCredentials(unsCreds, CRED_REG_INFO::UNS)) {
			if (m_Logger != NULL) {
				m_Logger->WarningLog("Failed to store UNS credentials");
			}
			printf("\nFailed to store UNS credentials\n");
		} else {
			if (m_Logger != NULL) {
				m_Logger->DebugLog("User Notification credentials were updated");
			}
		}
		delete unsCreds;
	}

	return 0;
}

//*****************************************************************************
// Name			:	ParseArguments
// Description	:	Parse the command line arguments.
//*****************************************************************************
unsigned long UserNotificationService::ParseArguments(int argc, const char **argv,
													  Credentials **unsCreds
#ifdef WITH_AMTPP
													  , Credentials **eacCreds
#endif
													  , bool &clear)
{
	Options::FormatNode RawFormat[] = {
		{"unsUser", false, true}, {"unsPass", false, true},
#ifdef WITH_AMTPP
		{"eacUser", false, true}, {"eacPass", false, true},
#endif
		{"cainfo", true, false}, {"capath", true, false},
		{"cert", true, false}, {"key", true, false},
		{"clear", false, false}};

	Options::Format format(RawFormat, sizeof(RawFormat)/sizeof(Options::FormatNode));

#ifdef _DEBUG_MOST
	format.addOption("clientip", true, true);
	format.addOption("serverip", true, true);
#endif
	Options options;

	if (!options.parse(argc, argv, format)) {
		printf("%s\n",options.errorStr());
		return ERROR_INVALID_PARAMETER;
	}

	*unsCreds = NULL;
#ifdef WITH_AMTPP
	*eacCreds = NULL;
#endif

	if (options.optionExists("clear")) {
		clear = true;
		return NO_ERROR;
	}

	if (options.optionExists("cert") || options.optionExists("cainfo") ||
		options.optionExists("key"))
	{
		*unsCreds = new Credentials;
#ifdef WITH_AMTPP
		*eacCreds = new Credentials;
#endif
		if (options.optionExists("cert")) {
			(*unsCreds)->certificate = options.getOption("cert");
#ifdef WITH_AMTPP
			(*eacCreds)->certificate = options.getOption("cert");
#endif
		}
		if (options.optionExists("cainfo")) {
			(*unsCreds)->cainfo = options.getOption("cainfo");
#ifdef WITH_AMTPP
			(*eacCreds)->cainfo = options.getOption("cainfo");
#endif
		}
		if (options.optionExists("key")) {
			(*unsCreds)->sslkey = options.getOption("key");
#ifdef WITH_AMTPP
			(*eacCreds)->sslkey = options.getOption("key");
#endif
		}
	}

	if (options.optionExists("unsUser")) {
		if (*unsCreds == NULL) {
			*unsCreds = new Credentials;
		}
		(*unsCreds)->username = options.getOption("unsUser");
		if (options.optionExists("unsPass")) {
			(*unsCreds)->password = options.getOption("unsPass");
		}
	} else if (options.optionExists("unsPass")) {
		return ERROR_INVALID_PARAMETER;
	}

#ifdef WITH_AMTPP
	if (options.optionExists("eacUser")) {
		if (*eacCreds == NULL) {
			*eacCreds = new Credentials;
		}
		(*eacCreds)->username = options.getOption("eacUser");
		if (options.optionExists("eacPass")) {
			(*eacCreds)->password = options.getOption("eacPass");
		}
	} else if (options.optionExists("eacPass")) {
		return ERROR_INVALID_PARAMETER;
	}
#endif
#ifdef _DEBUG_MOST
	if (options.optionExists("clientip")) {
		setClientIP(options.getOption("clientip"));
	}

	if (options.optionExists("serverip")) {
		setServerIP(options.getOption("serverip"));
	}
#endif
	return NO_ERROR;
}

#ifdef _DEBUG_MOST
//*****************************************************************************
// Name			: SetClientIP
// Description	: DEBUG ONLY - set remote AMT IP.
//*****************************************************************************
void UserNotificationService::setClientIP(const string &clip)
{
	m_UserNotificationThread.SetClientIP(clip);
#ifdef WITH_AMTPP
	m_PostureRetrievalThread.SetIP(clip);
#endif
}

//*****************************************************************************
// Name			: SetserverIP
// Description	: DEBUG ONLY - set remote UNS server IP.
//*****************************************************************************
void UserNotificationService::setServerIP(const string &svip)
{
	m_UserNotificationThread.SetServerIP(svip);
}
#endif
